#include "ClSocket.h"

#ifdef _WIN32
	#define MSG_NOSIGNAL	0
	#define	MSG_WAITALL		0
#else
	#include <sys/errno.h>
	#include <sys/ioctl.h>
	#include <arpa/inet.h>
#endif

#ifdef _DARWIN
	#define MSG_NOSIGNAL	0
#endif

#ifdef _WIN32
#else //Linux
    #include <netdb.h>
    #include <sys/socket.h>
    #include <netinet/in.h>
    #include <arpa/inet.h>
#endif

bool ClSocket::m_initialized = false;

ClSocket::ClSocket()
{
	m_socket = INVALID_SOCKET;
	m_socketStatus = 0;
	m_bBlocking = true;
}

ClSocket::ClSocket( SOCKET sock)
{
	setSocket( sock );
	m_bBlocking = true;
}

ClSocket::~ClSocket()
{
	setStatus( -1 );
}

int ClSocket::getData( char *buffer, int bufferSize )
{
	int readed, total = 0;

	while( total < bufferSize )
	{
		if( (readed = getStatus()) < 0 )
		{
			if ( total > 0 )
			{
				return total;
			}
			return readed;
		}

		readed = recv( m_socket, &buffer[total], bufferSize - total, MSG_NOSIGNAL|MSG_WAITALL );

		if( (readed == 0) || (readed == SOCKET_ERROR) )
		{
#ifdef _WIN32
			if( (readed == SOCKET_ERROR) && (WSAGetLastError () == WSAEWOULDBLOCK) )
#else			
			if( (readed == SOCKET_ERROR) && (errno == EAGAIN) )
#endif
			{
			    readed = 0;
			    Sleep( 10 );
			}
			else
			{
				setStatus( -1 );
				if ( total > 0 )
				{
					return total;
				}
			    return -1;
			}
		}
		total += readed;
	}
	return total;
}

int ClSocket::getLine( _TCHAR* buffer, int bufferSize )
{
	int i = 0, readed;
	_TCHAR c;

	while( i < bufferSize )
	{
		readed = getData( (char*)&c, sizeof( _TCHAR ) );
		if( readed < 0 )
		{
			buffer[0] = _T('\0');
			return readed;
		}
		if( (c == _T('\r')) || (c == _T('\n')) || (c == _T('\0')) )
		{
			break;
		}
		buffer[i] = c;
		i++;
	}
	buffer[i] = _T('\0');
	return i;
}

int ClSocket::getStatus( void )
{
	if( m_socket== INVALID_SOCKET )
	{
		return -1;
	}
	return m_socketStatus;
}

int ClSocket::sendData( const char *buffer, int bufferSize )
{
	int sended, total = 0;			
	if( bufferSize == 0 )
	{
		return getStatus();
	}
	
	while( total < bufferSize )
	{
		if( (sended = getStatus ()) < 0 )
		{
			return sended;
		}
		sended = send( m_socket, &buffer[total], bufferSize - total, MSG_NOSIGNAL );
		if( (sended == 0) || (sended == SOCKET_ERROR) )
		{
#ifdef _WIN32
			if( (sended == SOCKET_ERROR) && (WSAGetLastError () == WSAEWOULDBLOCK) )
#else			
			if( (sended == SOCKET_ERROR) && (errno == EAGAIN) )
#endif
			{
			    sended = 0;
			    Sleep( 10 );
			}
			else
			{
			    setStatus( -1 );
			    return -1;
			}
		}
		total += sended;
	}
	return total;
}

int ClSocket::sendLine( const _TCHAR* buffer, int buffSize )
{
	return sendData( (const char* )(buffer), buffSize*sizeof( _TCHAR ) );
}

void ClSocket::setStatus( int status )
{
	if( m_socketStatus < 0 )
	{
		return;
	}
	m_socketStatus = status;

	if( (status < 0) && (m_socket != INVALID_SOCKET) )
	{

#ifdef _WIN32
		closesocket( m_socket );
#else
		shutdown( m_socket, SHUT_RDWR );
		close( m_socket );
#endif
		m_socket = INVALID_SOCKET;
	}
}

void ClSocket::setSocket( SOCKET sock )
{
	m_socket = sock;
	m_socketStatus = 0;

#ifdef _DARWIN
	int p = 1;
	setsockopt( m_socket, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&p, sizeof(int) );
#endif
}

bool ClSocket::initSocketLibrary( void )
{

#ifdef _WIN32
	if( !m_initialized )
	{
		WORD socketVersionRequested;
		WSADATA wsaData;
		socketVersionRequested = MAKEWORD (2,0);
		if (WSAStartup (socketVersionRequested, &wsaData) != 0)
		{ 
			WSACleanup();
		}
		else
		{
			m_initialized = true;
		}
	}
#else
	m_initialized = true;
#endif
	
	return m_initialized;
}

void ClSocket::closeSocketLibrary( void )
{
#ifdef _WIN32
	WSACleanup();
#endif
	m_initialized = false;

}

void ClSocket::setBlocking( SOCKET sock, bool bBlocking )
{
	unsigned long uBlock = bBlocking? 0:1;
#ifdef _WIN32
	if ( SOCKET_ERROR == ioctlsocket( sock, FIONBIO, &uBlock ))
	{
		return;
	}
#else // Linux
	if (-1 == ioctl( sock, FIONBIO, &uBlock ))
	{
		return;
	}
#endif

	m_bBlocking = bBlocking;
}
